{% extends "base.html" %}

{% block title %}Bioregistry Metaregistry{% endblock %}

{% block container %}
    <div class="card">
        <div class="card-header">
            <div class="row">
                <div class="col-8"><h5 style="margin: 0">Metaregistry</h5></div>
                <div class="col-4 text-right">
                    <a href="{{ url_for("api.metaresources") }}">
                        JSON <i class="fas fa-download"></i>
                    </a>
                </div>
            </div>
        </div>
        <table id="table-entries" class="table table-striped table-hover ">
            <thead>
            <tr>
                <td>Name</td>
                <td>Metaprefix</td>
                <td>Description</td>
            </tr>
            </thead>
            {% for row in rows %}
                <tr>
                    <td>
                        <a href="{{ url_for('ui.metaresource', metaprefix=row.prefix) }}">{{ row.name }}</a>
                    </td>
                    <td class="text-nowrap"><code>{{ row.prefix }}</code></td>
                    <td>{% if row.description %}<small>{{ row.description }}</small>{% endif %}</td>
                </tr>
            {% endfor %}
        </table>
    </div>
{% endblock %}
